package com.goldencode.p2j.persist;

import com.goldencode.p2j.persist.hql.HQLParserTokenTypes;
import org.hibernate.QueryException;
import org.hibernate.engine.SessionFactoryImplementor;

import java.util.List;

public abstract class CommonSafeCompareFunction extends SafeCompareFunction implements HQLParserTokenTypes, SQLFunctionArgumentTypes
{
   protected abstract String equalRender();

   protected abstract String notEqualRender();

   protected abstract String falseNullableRender(String argStr);

   protected abstract String trueNullableRender(String arg);

   protected abstract String nullNullableRender(String arg);

   protected abstract String trueNotNullableRender(String arg);

   protected abstract String falseNotNullableRender(String arg);

   protected abstract String bothNotNullableRender(String arg1, String arg2);

   protected abstract String firstNotNullableRender(String arg1, String arg2);

   protected abstract String bothNullableRender(String arg1, String arg2);

   public final String render(List args, SessionFactoryImplementor factory) throws QueryException
   {
      if (args.size() != 4) {
         throw new QueryException("Function requires four arguments");
      }
      String[] argStrs = new String[4];
      for (int i = 0; i < 4; i++)
      {
         argStrs[i]= (String)args.get(i);
      }
      ArgType[] types = new ArgType[2];
      for (int i = 0; i < 2; i++)
      {
         types[i]= string2ArgType(argStrs[2 + i]);
      }
      String result = null;
      switch (types[0])
      {
         case NULL:
            switch (types[1])
            {
               case NULL:
                  result = equalRender();
                  break;
               case TRUE:
               case FALSE:
               case NOT_NULLABLE:
                  result = notEqualRender();
                  break;
               case NULLABLE:
                  result = nullNullableRender(argStrs[1]);
                  break;
            }
            break;
         case TRUE:
            switch (types[1])
            {
               case TRUE:
                  result = equalRender();
                  break;
               case NULL:
               case FALSE:
                  result = notEqualRender();
                  break;
               case NOT_NULLABLE:
                  result = trueNotNullableRender(argStrs[1]);
                  break;
               case NULLABLE:
                  result = trueNullableRender(argStrs[1]);
                  break;
            }
            break;
         case FALSE:
            switch (types[1])
            {
               case FALSE:
                  result = equalRender();
                  break;
               case NULL:
               case TRUE:
                  result = notEqualRender();
                  break;
               case NOT_NULLABLE:
                  result = falseNotNullableRender(argStrs[1]);
                  break;
               case NULLABLE:
                  result = falseNullableRender(argStrs[1]);
                  break;
            }
            break;
         case NOT_NULLABLE:
            switch (types[1])
            {
               case NULL:
                  result = notEqualRender();
                  break;
               case TRUE:
                  result = trueNotNullableRender(argStrs[0]);
                  break;
               case FALSE:
                  result = falseNotNullableRender(argStrs[0]);
                  break;
               case NOT_NULLABLE:
                  result = bothNotNullableRender(argStrs[0], argStrs[1]);
                  break;
               case NULLABLE:
                  result = firstNotNullableRender(argStrs[0], argStrs[1]);
                  break;
            }
            break;
         case NULLABLE:
            switch (types[1])
            {
               case NULL:
                  result = nullNullableRender(argStrs[0]);
                  break;
               case TRUE:
                  result = trueNullableRender(argStrs[0]);
                  break;
               case FALSE:
                  result = falseNullableRender(argStrs[0]);
                  break;
               case NOT_NULLABLE:
                  result = firstNotNullableRender(argStrs[1], argStrs[0]);
                  break;
               case NULLABLE:
                  result = bothNullableRender(argStrs[0], argStrs[1]);
                  break;
            }
            break;
      }
      return result;
   }

   private static ArgType string2ArgType(String argStr)
   {
      ArgType argType;
      if (argStr.equals("null"))
      {
         argType = ArgType.NULL;
      }
      else if (argStr.equals("true"))
      {
         argType = ArgType.TRUE;
      }
      else if (argStr.equals("false"))
      {
         argType = ArgType.FALSE;
      }
      else if (argStr.equals("not_nullable"))
      {
         argType = ArgType.NOT_NULLABLE;
      }
      else //if (argStr.equals("nullable"))
      {
         argType = ArgType.NULLABLE;
      }
      return argType;
   }
}
